#!/usr/bin/lua5.1
-- TCP-to-serial Lua application, adapted for IPAM-400 Yocto
-- platform from the flexa/barionet version

-- Packages import
rs232 = require("luars232")
local socket = require("socket")
require("uci")


-- Uci context instance
x = uci.cursor()
SERIAL = {
	
	PORT_NAME 		= x:get("io-tunnel.serial_params.port_name"),
	BAUD 			= tonumber(x:get("io-tunnel.serial_params.baud")),		
	DATA_BITS 		= tonumber(x:get("io-tunnel.serial_params.data_bits")),					
	PARITY 			= x:get("io-tunnel.serial_params.parity"),				
	STOP_BITS		= tonumber(x:get("io-tunnel.serial_params.stop_bits")),					
	FLOW_CONTROL	= x:get("io-tunnel.serial_params.flow_control"),	
			
}

SERVER = {

	MODE 			= x:get("io-tunnel.application.mode"),			
	SERVER_IP		= x:get("io-tunnel.application.ip"),	
	PORT			= tonumber(x:get("io-tunnel.application.port")),

}

-- Specify serial connection parameters
baud_rates = {
	[9600] 		= rs232.RS232_BAUD_9600,
	[19200]		= rs232.RS232_BAUD_19200,
	[38400]		= rs232.RS232_BAUD_38400,
	[57600]		= rs232.RS232_BAUD_57600,
	[115200]	= rs232.RS232_BAUD_115200,
}

data_bits = {
	[6] 		= rs232.RS232_DATA_6,
	[7] 		= rs232.RS232_DATA_7,
	[8] 		= rs232.RS232_DATA_8,
}

stop_bits = {
	[1] 		= rs232.RS232_STOP_1,
	[2] 		= rs232.RS232_STOP_2,
}

parity_bits = {
	["NONE"] 	= rs232.RS232_PARITY_NONE,
	["ODD"]		= rs232.RS232_PARITY_ODD,
	["EVEN"]	= rs232.RS232_PARITY_EVEN,
}

flow_control = {
	["OFF"]		= rs232.RS232_FLOW_OFF,
	["HW"]		= rs232.RS232_FLOW_HW,
	["XONXOFF"]	= rs232.RS232_FLOW_XON_XOFF,
}

--  Setup
function serial_setup()

	local out = io.stderr

	e, p = rs232.open(SERIAL.PORT_NAME)
	if e ~= rs232.RS232_ERR_NOERROR then									-- handle error
		out:write(string.format("can't open serial port '%s', error: '%s'\n",
				port_name, rs232.error_tostring(e)))
		return
	end


	assert(p:set_baud_rate(baud_rates[SERIAL.BAUD]) == rs232.RS232_ERR_NOERROR)
	assert(p:set_data_bits(data_bits[SERIAL.DATA_BITS]) == rs232.RS232_ERR_NOERROR)
	assert(p:set_parity(parity_bits[SERIAL.PARITY]) == rs232.RS232_ERR_NOERROR)
	assert(p:set_stop_bits(stop_bits[SERIAL.STOP_BITS]) == rs232.RS232_ERR_NOERROR)
	assert(p:set_flow_control(flow_control[SERIAL.FLOW_CONTROL])  == rs232.RS232_ERR_NOERROR)

	out:write(string.format("OK, port open with values '%s'\n", tostring(p)))

end

--CLIENT FUNCTION
function client_f(host, port)

	tcp = assert(socket.tcp())

	tcp:connect(host, port);

end

--SERVER FUNCTION
function server_f(port)

    -- create a TCP socket and bind it to the local host, at any port
	local server = assert(socket.bind("*", port)) 							
	-- find out which port the OS chose for us
	local ip, port = server:getsockname()									
	-- print a message informing what's up
	print("Please telnet to localhost on port " .. port)					
	print("After connecting, you have 30s to enter a line to be echoed")
	
	-- wait for a connection from any client
	client = server:accept()												
	client:settimeout(30)

end

--SERIAL RECEIVE FUNCTION
function serial_receive()

	local e, data_read = p:read(1,100)
	--assert(e == rs232.RS232_ERR_NOERROR)
	if data_read ~= nil then
		server_send(data_read)
	end
		
end

--SERIAL TRANSMIT FUNCTION
function serial_transmit(data_send)
	
	if data_send ~= nil then
		e, len_written = p:write(data_send)
	end
	assert(e == rs232.RS232_ERR_NOERROR)

end

--TCP RECEIVE FUNCTION
function tcp_receive()

	if mode == "SERVER" then
		client:settimeout(0.1)
		local line, err = client:receive(1)
		serial_transmit(line)
	else 
		tcp:settimeout(0.1)
		local s, status, partial = tcp:receive(1)
		serial_transmit(s or partial)
	end	
		
end

--TCP SEND FUNCTION
function server_send(data)

	if mode == "SERVER" then
		client:send(data)
	else 
		tcp:send(data);
	end
end


serial_setup()

mode, port = SERVER.MODE, SERVER.PORT

print("MODE: " .. mode .. "\n")

if mode == "CLIENT" then
	host = SERVER.SERVER_IP
	print("Host to connect: ".. host .."\n" .. "Used port: " .. port .. "\n")
	client_f(host,port)

elseif mode == "SERVER" then

	print("Used port: " .. port .."\n")
	server_f(port)
end	

-- Start tunneling application
while true do
	serial_receive()
	tcp_receive()
end

-- done with client, close the object
tcp:close()
client:close()														
